package checkers.metaquals;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

// The framework can reflectively query the @TypeQualifiers, SubtypeOf, and
// QualifierRoot, then that information to construct the qualifiers hierarchy.

/**
 * An annotation that lists the type qualifiers supported by the annotated
 * {\code Checker}.
 *
 * <p>
 * Example:
 * <pre>@TypeQualifiers( { Nullable.class, NonNull.class } )
 * public class NonNullChecker extends BaseTypeChecker { ... }
 * </pre>
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target ( { ElementType.TYPE } )
public @interface TypeQualifiers {
    Class<?>[] value();
}
